# \label{c5_poe_angles1}
# \label{c5_poe_angles2}
# Programs 5.4 and 5.5, pages 155-6

  open(IN, "Poe_4_Stories.txt") or die;
  $nstory = -1;
  
  # Step 1: Compute the term frequencies
  while (<IN>) {
    chomp;
    if ( /<TITLE>(.*)<\/TITLE>/ ) {
      $title = $1; 
      ++$nstory; 
      print "Title of story $nstory = $title\n";
    } else {
      if ($title eq '') { print "NULL TITLE\n"; } 
      $_ = lc;   # Change letters to lower case
      s/--/ /g;  # Remove dashes
      s/ - / /g; # Remove dashes
      s/[,.";!()?:_\[\]]//g; # Remove non-apostrophes
      s/\s+/ /g; # Replace multiple spaces with one space
      s/^\s+//g; # Remove spaces at the start of a line
      @words = split(/ /);
      
      foreach $word (@words) {
        if ($word =~ /^'?(.*?)'?$/) { # Must be non-greedy
          $word = $1; # Remove initial and final apostrophes
          ++$tf[$nstory]{$word};
          ++$granddict{$word};
        }
      }      
    }
  }

  # Step 2: Compute the document frequencies
  foreach $word ( sort keys %granddict ) {
    $sum = 0;
    for $i ( 0 .. $#tf ) {
      if ( $tf[$i]{$word} > 0 ) {
        ++$sum;
      }
      $df{$word} = $sum;
    }
  }


  # Step 3: Compute the tf-idf weights
  $n = $#tf + 1; # number of stories
  foreach $word ( sort keys %granddict ) {
    for $i ( 0 .. $#tf ) {
      $weight[$i]{$word} = $tf[$i]{$word}*log($n/$df{$word})/log(2);
    }
  }

  # Step 4: Normalize the columns of weights
  for $i ( 0 .. $#tf ) {
    $len2 = 0;
    foreach $word ( sort keys %granddict ) {
      $len2 += $weight[$i]{$word}**2; 
    }  
    $len = sqrt($len2);
    foreach $word ( sort keys %granddict ) {
      $unit[$i]{$word} = $weight[$i]{$word}/$len; 
    }
  }

  # Step 5: Compute cosine similarities between each pair of stories
  for $i ( 0 .. $#tf ) {
    for $j (0 .. $#tf) {
      $sum = 0;
      foreach $word ( sort keys %granddict ) {
        $sum += $unit[$i]{$word} * $unit[$j]{$word}; 
      }
      $cosine[$i][$j] = $sum;
    }
  }

  # Print out the cosine similarities
  for $i ( 0 .. $#tf ) {
    for $j ( 0 .. $#tf ) {
      printf "%.4f ", $cosine[$i][$j];
    }
    print "\n";
  }

   close(IN);