# \label{c6_concord}
# \label{c6_onleft}
# \label{c6_byleftwords}
# \label{c6_byrightwords}
# \label{c6_concord_sort_print}

# Program 6.1 and Code Samples 6.3, 6.7, 6.8, 6.9, 6.10, pages 172-3 and 180-181

  # EXAMPLE: perl concordance.pl regex radius ordinal
  open (FILE, "Call of the Wild.txt") or die("File not found");
  
  $/ = ""; # Paragraph mode for while loop
  $target = "($ARGV[0])"; print "Target=$target\n\n";
  $radius = $ARGV[1];
  $width = 2*$radius; 
  $ordinal = $ARGV[2];
  
  $count = 0;
  while (<FILE>) {
    chomp;
    s/\n/ /g; # Replace newlines by spaces
    s/\b--\b/ -- /g; # Add spaces around dashes adjacent to words
    
    while ( $_ =~ /$target/gi ) {
      $match = $1;
      $pos = pos;
      $start = $pos - $radius - length($match);
      
      # Extracts are padded with spaces if needed
      if ($start < 0) {
        $extract = substr($_, 0, $width+$start+length($match));
        $extract = (" " x -$start) . $extract;
      } else {
        $extract = substr($_, $start, $width+length($match));
        $deficit = $width+length($match) - length($extract);
        if ($deficit > 0) {
          $extract .= (" " x $deficit);
        }
      }
      $lines[$count] = $extract;
      ++$count;
    }
  }

  close(FILE);


# Code Sample 6.3, page 173

  sub removePunctuation {
    # USAGE: $unpunctuated = removePunctuation($string);
    my $string = $_[0];
    $string =  lc($string);   # Convert to lowercase
    $string =~ s/[^-a-z ]//g; # Remove non-alphabetic characters
    $string =~ s/--+/ /g;     # Replace 2+ hyphens with a space
    $string =~ s/-//g;        # Remove hyphens
    $string =~ s/\s+/ /g;     # Replace whitespaces with a space
    return($string);
  } 

# Code Sample 6.7, page 180

  sub onLeft {
    # USAGE: $word = onLeft($string, $radius, $ordinal);
    my $left = substr($_[0], 0, $_[1]);    
    $left = removePunctuation($left);
    my @word = split(/\s+/, $left);  
    return($word[-$_[2]]);
  }

# Code Sample 6.8, page 180

  sub byLeftWords {
    my $left_a = onLeft($a, $radius, $ordinal); 
    my $left_b = onLeft($b, $radius, $ordinal); 
    lc($left_a) cmp lc($left_b);
  }

# Code Sample 6.9, page 181

  sub onRight {
    # USAGE: $word = onRight($string, $radius, $ordinal);
    my $right = substr($_[0], -$_[1]); 
    $right = removePunctuation($right);
    $right =~ s/^\s+//;      # Remove initial space
    my @word = split(/\s+/, $right);  
    return($word[$_[2]-1]);
  }

  sub byRightWords {
    my $right_a = onRight($a, $radius, $ordinal); 
    my $right_b = onRight($b, $radius, $ordinal); 
    lc($right_a) cmp lc($right_b);
  }


# Code Sample 6.10, page 181

  $line_number = 0;
  foreach $x (sort byLeftWords @lines) {
    ++$line_number;
    printf "%5d", $line_number;
    print " $x\n";
  }

  $line_number = 0;
  foreach $x (sort byRightWords @lines) {
    ++$line_number;
    printf "%5d", $line_number;
    print " $x\n";
  }